const { trim } = require('./utils');

const graphqlTypes = {
  string: 'String',
  boolean: 'Boolean',
  number: 'Float',
  integer: 'Int',
};

function graphqlInput(value) {
  const {
    get, set, input, required,
  } = value;

  const prefix = get || set;

  return input
    ? `${prefix}(input: ${input}${required ? '!' : ' = {}'})`
    : prefix;
}

function graphqlOutput(value, defns) {
  if (defns[value.resp]) {
    const schema = defns[value.resp];
    const target = graphqlTypes[schema] || schema;

    return `[${target}]`;
  }

  return value.resp;
}

function graphqlType(info, value) {
  const {
    type, field, schema, required,
  } = value;

  const suffix = required ? '!' : '';
  const target = graphqlTypes[schema] || schema;

  if (type === 'list') {
    return `${field}: [${target}]${suffix}`;
  }

  return `${field}: ${target}${suffix}`;
}

module.exports = {
  blueprint({ calls, defns }) {
    const Query = calls.filter(call => call.get);
    const Mutation = calls.filter(call => call.set);

    const definitions = { Query, Mutation };

    return Object.keys(definitions).reduce((prev, cur) => {
      if (definitions[cur] && definitions[cur].length) {
        prev.push(trim(`
          extend type ${cur} {
          ${definitions[cur].map(call => `  ${graphqlInput(call)}: ${graphqlOutput(call, defns)}`).join('\n')}
          }
        `));
      }

      return prev;
    }, []).join('\n');
  },
  definition(pkgInfo, { name, props, target }) {
    return trim(`
      ${target} ${name} {${props ? (props.length ? '\n' : '')
      + props.map(prop => `  ${graphqlType(pkgInfo, prop)}\n`).join('') : ''}}
    `);
  },
  enumeration(pkgInfo, { schema, values, offset }) {
    return trim(`
      enum ${schema || `ENUM_${offset}`} {${values ? (values.length ? '\n' : '')
        + values.map(value => `  ${value}\n`).join('') : ''}}
    `);
  },
};
